/////////////////////////////////////////////////////////////////////////////
//
// Written by Igor Tolmachev (support@itsamples.com)
// Copyright (c) 2017 Igor Tolmachev.
//
// This code may be used in compiled form in any way you desire. This
// source file may be redistributed by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. 
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage to you or your
// computer whatsoever. It's free, so don't hassle me about it.
//
// Beware of bugs.
//
// MyMAC.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <iphlpapi.h>
#include <netcon.h>
#include "validmacs.h"

#pragma warning(disable: 4996) // _CRT_SECURE_NO_WARNINGS: swprintf

#pragma comment(lib, "iphlpapi.lib")

#define REG_NAME_SIZE   260
#define REG_BUFFER_SIZE 384

void RandomizeMAC(char *pszMacAddress)
{
    srand(GetTickCount());

    _snprintf(pszMacAddress, 6, "%06X", rand() % numMacs);
    for (int i = 3; i < 6; i++) 
        _snprintf(&pszMacAddress[i*2], 2, "%02X", rand() & 0xFF);

    pszMacAddress[12] = 0;
}

void PrintRandomMAC()
{
    srand(GetTickCount());

    char szMacAddress[16];
    _snprintf(szMacAddress, 6, "%06X", rand() % numMacs);

    for (int i = 3; i < 6; i++)
        _snprintf(&szMacAddress[i*2], 2, "%02X", rand() & 0xFF);

    szMacAddress[12] = 0;
    printf("Random MAC Address:\t%s", szMacAddress);
}

BOOL IsValidMAC(char * pszMacAddress)
{
    if(strlen(pszMacAddress) != 12)
        return false;

    for(int i = 0; i < 12; i++)
    {
        if((pszMacAddress[i] < '0' || pszMacAddress[i] > '9')
            && (pszMacAddress[i] < 'a' || pszMacAddress[i] > 'f')
            && (pszMacAddress[i] < 'A' || pszMacAddress[i] > 'F'))
                return false;
    }

    return true;
}

void ResetAdapter(char *pszAdapterName)
{
    GUID guid = {0xBA126AD1,0x2166,0x11D1,0};
    memcpy(guid.Data4, "\xB1\xD0\x00\x80\x5F\xC1\x27\x0E", 8);

    wchar_t *pBuffer = new wchar_t[strlen(pszAdapterName)+1];
    memset(pBuffer, 0, sizeof(pBuffer));

    void (__stdcall *NcFreeNetConProperties) (NETCON_PROPERTIES *);
    HMODULE hNetShell = LoadLibrary("Netshell.dll");
    if(!hNetShell) 
    {
        printf("Couldn't load Netshell.dll\n");
        return;
    }

    NcFreeNetConProperties = (void (__stdcall *)(struct tagNETCON_PROPERTIES *))GetProcAddress(hNetShell, "NcFreeNetconProperties");
    if (!NcFreeNetConProperties)
    {
        printf("Couldn't load NcFreeNetconProperties function\n");
        return;
    }

    int nLen = strlen(pszAdapterName);
    for(int i = 0; i <= nLen; i++)
    {
        pBuffer[i] = pszAdapterName[i];
    }

    CoInitialize(NULL);
    INetConnectionManager *pNCM = NULL;
    HRESULT hr = ::CoCreateInstance(guid, NULL, CLSCTX_ALL, __uuidof(INetConnectionManager), (void**)&pNCM);
    if(SUCCEEDED(hr) && pNCM)
    {
        IEnumNetConnection *pENC = NULL;
        pNCM->EnumConnections(NCME_DEFAULT, &pENC);
        if(SUCCEEDED(hr) && pENC)
        {
            INetConnection *pNC = NULL;
            ULONG nFetched = 0;
            NETCON_PROPERTIES *pNCP = NULL;

            while(1)
            {
                pENC->Next(1, &pNC, &nFetched);
                if(nFetched && pNC)
                {
                    pNC->GetProperties(&pNCP);
                    if(pNCP)
                    {
                        if(wcscmp(pNCP->pszwName, pBuffer) == 0)
                        {
                            pNC->Disconnect();
                            pNC->Connect();
                        }

                        NcFreeNetConProperties(pNCP);
                    }
                }
                else
                    break;
            }

            pENC->Release();
        }
        pNCM->Release();
    }

    FreeLibrary(hNetShell);
    CoUninitialize();
}

void SetMAC(char *pszAdapterName, char *pszNewMAC)
{
    HKEY hKey = NULL;
    HKEY hListKey = NULL;
    BOOL bFound = FALSE;
    DWORD dwType = NULL;
    LONG nQuery = NULL;
    DWORD dwKeySize = REG_BUFFER_SIZE;
    FILETIME ftLastWriteTime;

    char szTargetAdapterGuid[REG_NAME_SIZE];
    char szTargetAdapterName[REG_NAME_SIZE];
    char szKeyNameBuffer[REG_BUFFER_SIZE];
    char szKeyValueBuffer[REG_BUFFER_SIZE];

    int i = 0;
    memset(szTargetAdapterGuid, 0, REG_NAME_SIZE);
    memset(szTargetAdapterName, 0, REG_NAME_SIZE);
    if(pszAdapterName && strstr(pszAdapterName, "{") && strstr(pszAdapterName, "}"))
    {
        strcpy(szTargetAdapterGuid, pszAdapterName);
        memset(szKeyNameBuffer, 0, REG_BUFFER_SIZE);
        _snprintf(szKeyNameBuffer, REG_BUFFER_SIZE, "SYSTEM\\CurrentControlSet\\Control\\Network\\{4D36E972-E325-11CE-BFC1-08002BE10318}\\%s\\Connection", szTargetAdapterGuid);
        nQuery = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKeyNameBuffer, 0, KEY_READ, &hListKey);
        if(nQuery == ERROR_SUCCESS && hListKey)
        {
            dwKeySize = REG_BUFFER_SIZE;
            memset(szKeyValueBuffer, 0, REG_BUFFER_SIZE);
            nQuery = RegQueryValueEx(hListKey, "Name", 0, &dwType, (LPBYTE)szKeyValueBuffer, &dwKeySize);
            if(nQuery == ERROR_SUCCESS)
            {
                strcpy(szTargetAdapterName, szKeyValueBuffer);
                bFound = TRUE;
            }
            RegCloseKey(hListKey);
        }
        else
        {
            printf("Failed to open adapter GUID key\n");
            return;
        }
    }
    else
    {
        strcpy(szTargetAdapterName, pszAdapterName);
        nQuery = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SYSTEM\\CurrentControlSet\\Control\\Network\\{4D36E972-E325-11CE-BFC1-08002BE10318}", 0, KEY_READ, &hListKey);
        if(nQuery == ERROR_SUCCESS && hListKey)
        {
            while (RegEnumKeyEx(hListKey, i++, szKeyNameBuffer, &dwKeySize, 0, NULL, NULL, &ftLastWriteTime) == ERROR_SUCCESS) 
            {
                memset(szKeyValueBuffer, 0, REG_BUFFER_SIZE);
                _snprintf(szKeyValueBuffer, REG_BUFFER_SIZE, "%s\\Connection", szKeyNameBuffer);

                hKey = NULL;
                nQuery = RegOpenKeyEx(hListKey, szKeyValueBuffer, 0, KEY_READ, &hKey);
                if(hKey && nQuery == ERROR_SUCCESS)
                {
                    dwKeySize = REG_BUFFER_SIZE;
                    memset(szKeyValueBuffer, 0, REG_BUFFER_SIZE);
                    if(RegQueryValueEx(hKey, "Name", 0, &dwType, (LPBYTE)szKeyValueBuffer, &dwKeySize) == ERROR_SUCCESS && strcmp(szKeyValueBuffer, pszAdapterName) == 0)
                    {
                        strcpy(szTargetAdapterGuid, szKeyNameBuffer);

                        bFound = TRUE;
                        break;
                    }
                    RegCloseKey(hKey);
                }

                dwKeySize = REG_BUFFER_SIZE;
            }
            RegCloseKey(hListKey);
        }
        else
        {
            printf("Failed to open adapter list key\n");
            return;
        }
    }

    if(!bFound) 
    {
        printf("Could not find adapter name '%s'.\nPlease make sure this is the name you gave it in Network Connections.\n", pszAdapterName);
        return;
    }

    hListKey = NULL;
    nQuery = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SYSTEM\\CurrentControlSet\\Control\\Class\\{4D36E972-E325-11CE-BFC1-08002bE10318}", 0, KEY_READ, &hListKey);
    if(nQuery == ERROR_SUCCESS && hListKey)
    {
        i = 0;
        dwKeySize = REG_BUFFER_SIZE;
        memset(szKeyValueBuffer, 0, REG_BUFFER_SIZE);
        char szNetInstanceBuffer[REG_BUFFER_SIZE];
        while(RegEnumKeyEx(hListKey, i++, szKeyNameBuffer, &dwKeySize, 0, NULL, NULL, &ftLastWriteTime) == ERROR_SUCCESS)
        {
            hKey = NULL;
            dwType = NULL;
            nQuery = RegOpenKeyEx(hListKey, szKeyNameBuffer, 0, KEY_READ|KEY_SET_VALUE, &hKey);
            if(nQuery == ERROR_SUCCESS && hKey)
            {

                dwKeySize = REG_BUFFER_SIZE;
                memset(szNetInstanceBuffer, 0, REG_BUFFER_SIZE);
                if((RegQueryValueEx(hKey, "NetCfgInstanceId", 0, &dwType, (LPBYTE)szNetInstanceBuffer, &dwKeySize) == ERROR_SUCCESS) && (strcmp(szNetInstanceBuffer, szTargetAdapterGuid) == 0)) 
                {
                    RegSetValueEx(hKey, "NetworkAddress", 0, REG_SZ, (LPBYTE)pszNewMAC, strlen(pszNewMAC) + 1);
                    printf("Resetting adapter...\n");
                    ResetAdapter(szTargetAdapterName);
                    printf("Done");
                }
                RegCloseKey(hKey);
            }

            dwKeySize = REG_BUFFER_SIZE;
        }
        RegCloseKey(hListKey);
    }
    else
    {
        printf("Failed to open adapter list key in Phase 2\n");
        return;
    }
}

void ShowHelp() 
{
    printf("Usage: mymac [options] [mac-address]\n\n");
    printf("options:\n");
    printf("\t[adapter]\tAdapter name from Network Connections or its GUID\n");
    printf("\t-all\t\tPrints all available adapters\n");
    printf("\t-rnd\t\tPrints a random MAC address\n");
    printf("\t-set\t\tSets new MAC address\n");
    printf("\t-r\t\tRestores the original MAC address.\n");
    printf("\t-help\t\tShows this screen\n");
    printf("examples:\n");
    printf("\tmymac -all\n");
    printf("\tmymac -set \"Local Area Connection\"\n");
    printf("\tmymac -set \"Local Area Connection\" -r\n");
    printf("\tmymac -set \"Local Area Connection\" 000F504DD7FA\n");
    printf("\tmymac -set \"{B6CAA8D7-9074-4A3C-87F4-DB769F6256F0}\"\n");
}

void GetMAC(BOOL bAll)
{
    DWORD dwMACaddress = 0;
    IP_ADAPTER_INFO AdapterInfo[16];
    DWORD dwBufLen = sizeof(AdapterInfo);

    DWORD dwStatus = GetAdaptersInfo(AdapterInfo, &dwBufLen);
    if(dwStatus == ERROR_SUCCESS)
    {
        PIP_ADAPTER_INFO pAdapterInfo = AdapterInfo;
        while(pAdapterInfo)
        {
            dwMACaddress = pAdapterInfo->Address[5] + pAdapterInfo->Address[4] * 256 +
                pAdapterInfo->Address[3] * 256 * 256 +
                pAdapterInfo->Address[2] * 256 * 256 * 256;

            if(bAll)
            {
                printf("Adapter ID:\t%s\nDescription:\t%s\nMAC Address:\t%02X-%02X-%02X-%02X-%02X-%02X\n",
                    pAdapterInfo->AdapterName,
                    pAdapterInfo->Description,
                    pAdapterInfo->Address[0], pAdapterInfo->Address[1], pAdapterInfo->Address[2], pAdapterInfo->Address[3], pAdapterInfo->Address[4], pAdapterInfo->Address[5]);

                printf("Adapter Type:\t");
                switch(pAdapterInfo->Type) 
                {
                case MIB_IF_TYPE_OTHER:
                    printf("Other\n");
                    break;
                case MIB_IF_TYPE_ETHERNET:
                    printf("Ethernet\n");
                    break;
                case MIB_IF_TYPE_TOKENRING:
                    printf("Token Ring\n");
                    break;
                case MIB_IF_TYPE_FDDI:
                    printf("FDDI\n");
                    break;
                case MIB_IF_TYPE_PPP:
                    printf("PPP\n");
                    break;
                case MIB_IF_TYPE_LOOPBACK:
                    printf("Lookback\n");
                    break;
                case MIB_IF_TYPE_SLIP:
                    printf("Slip\n");
                    break;
                case 71: // IF_TYPE_IEEE80211
                    printf("IEEE 802.11\n");
                    break;
                default:
                    printf("Unknown type %ld\n", pAdapterInfo->Type);
                    break;
                }

                printf("IP Address: \t%s\n", pAdapterInfo->IpAddressList.IpAddress.String);
                printf("IP Mask: \t%s\n", pAdapterInfo->IpAddressList.IpMask.String);
                printf("Gateway: \t%s\n", pAdapterInfo->GatewayList.IpAddress.String);

                if(pAdapterInfo->DhcpEnabled)
                    printf("DHCP Server: \t%s\n", pAdapterInfo->DhcpServer.IpAddress.String);

                if(pAdapterInfo->HaveWins) 
                {
                    printf("Primary Wins Server:    %s\n", pAdapterInfo->PrimaryWinsServer.IpAddress.String);
                    printf("Secondary Wins Server:  %s\n", pAdapterInfo->SecondaryWinsServer.IpAddress.String);
                }
            }
            else
                printf("MAC Address:\t%02X-%02X-%02X-%02X-%02X-%02X\n",
                    pAdapterInfo->Address[0], pAdapterInfo->Address[1], pAdapterInfo->Address[2], pAdapterInfo->Address[3], pAdapterInfo->Address[4], pAdapterInfo->Address[5]);

            printf("\n\n");

            pAdapterInfo = pAdapterInfo->Next;
        }
    }
}

int main(int argc, char* argv[])
{
    if(argc > 1)
    {
        char szNewMac[16];
        char szAdapter[REG_NAME_SIZE];

        BOOL bInSet = FALSE;
        BOOL bReset = FALSE;
        memset(szNewMac, 0, 16);
        memset(szAdapter, 0, REG_NAME_SIZE);
        for(int i = 0; i < argc; i++)
        {
            if(!strcmp(argv[i], "/all") || !strcmp(argv[i], "-all") || !strcmp(argv[i], "-a"))
            {
                GetMAC(TRUE);
                return 0;
            }
            else if(!strcmp(argv[i], "/rnd") || !strcmp(argv[i], "-rnd"))
            {
                PrintRandomMAC();
                return 0;
            }
            else if(!strcmp(argv[i], "/set") || !strcmp(argv[i], "-set") && (i < argc - 1))
            {
                bInSet = TRUE;
                strcpy(szAdapter, argv[++i]);
            }
            else if(!strcmp(argv[i], "/reset") || !strcmp(argv[i], "-r"))
            {
                bReset = TRUE;
            }
            else if(IsValidMAC(argv[i]))
            {
                strncpy(szNewMac, argv[i], 12);
            }
            else if(!strcmp(argv[i], "/h") || !strcmp(argv[i], "-help") || !strcmp(argv[i], "--help"))
            {
                ShowHelp();
                return 0;
            }
        }

        if(bInSet)
        {
            if(bReset)
                strcpy(szNewMac, "");
            else if(strlen(szNewMac) < 12)
                RandomizeMAC(szNewMac);
            if(strlen(szAdapter) < 2)
                printf("Selected adapter is not valid");

            if(strlen(szNewMac))
                printf("Changing MAC address (%s) for %s...\n", szNewMac, szAdapter);
            else
                printf("Setting factory default MAC address for %s...\n", szAdapter);

            SetMAC(szAdapter, szNewMac);
        }
        else
        {
            ShowHelp();
        }
    }
    else
        ShowHelp();

    return 0;
}

